"""Test Result Module of Unit Test Package

@(#) testResult.py 1.7@(#)"""

import time
from testFailure import TestFailure


class TestResult:
    """Holder and reporter of test results."""

    def __init__ (self, aTest):
	"""Initialize with a test."""
	self.test = aTest
	self.failures = []
	self.errors = []

    def start (self):
	"""Note the start time."""
	self.startTime = time.time ()

    def stop (self):
	"""Note the stop time."""
	self.stopTime = time.time ()

    def wasSuccessful (self):
	"""Return true if successful."""
	return self.hasNoFailures () and self.hasNoErrors ()

    def hasNoFailures (self):
	"""Return true if there are no failures."""
	return len (self.failures) == 0

    def hasNoErrors (self):
	"""Return true if there are no errors."""
	return len (self.errors) == 0

    def failureIn (self, aTestCase, moreInfo = None):
	"""Record a failure signaled by an exception in a test case."""
	self.failures.append (TestFailure (aTestCase, moreInfo))

    def errorIn (self, aTestCase, moreInfo = None):
	"""Record a error signaled by an exception in a test case."""
	self.errors.append (TestFailure (aTestCase, moreInfo))

    def printOn (self, aStream):
	"""Print the test results on a stream."""
	self.printHeadersOn (aStream)
	self.printErrorsOn (aStream)
	self.printFailuresOn (aStream)

    def printHeadersOn (self, aStream):
	"""Print the test result header on a stream."""
	aStream.write ("Test results for %s\n" % self.test.getName ())
	aStream.write ("    Start time: %s\n" % time.ctime (self.startTime))
	aStream.write ("    Stop time: %s\n\n" % time.ctime (self.stopTime))

    def printErrorsOn (self, aStream):
	"""Print errors on a stream."""
	for anError in self.errors:
	    aStream.write ("    ")
	    self.printError (anError, aStream)
	    aStream.write ("\n")

    def printError (self, anError, aStream):
	"""Print an error on a stream."""
	aStream.write ("Error-\t")
	anError.printOn (aStream)

    def printFailuresOn (self, aStream):
	"""Print failures on a stream."""
	for aFailure in self.failures:
	    aStream.write ("    ")
	    self.printFailureOn (aFailure, aStream)
	    aStream.write ("\n")

    def printFailureOn (self, aFailure, aStream):
	"""Print an failure on a stream."""
	aStream.write ("Failure-\t")
	aFailure.printOn (aStream)

    def formatTimeForSummary (self, dateTime):
	"""Format a time for summary statistics."""
	return time.strftime ("%Y" "%m" "%d\t%H:%M:%S", time.localtime (dateTime))

    def printSummaryOn (self, aStream):
	"""Print summary statistics on a stream."""
	aStream.write ("%s\t%s\t%s\t%d\t%d\t%d\n" %
		(self.test.getName (),
		self.formatTimeForSummary (self.startTime),
		self.formatTimeForSummary (self.stopTime),
		self.test.getTestCaseCount (),
		len (self.failures),
		len (self.errors)))
